﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Selection;

namespace HeuristicLab.Optimization.Operators.LCS {
  [Item("PreservingRightReducer", "")]
  [StorableClass]
  public sealed class PreservingRightReducer : Reducer, IReducer {
    [StorableConstructor]
    private PreservingRightReducer(bool deserializing) : base(deserializing) { }
    private PreservingRightReducer(PreservingRightReducer original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new PreservingRightReducer(this, cloner);
    }
    public PreservingRightReducer() : base() { }

    protected override List<IScope> Reduce(List<IScope> scopes) {
      IScope rightChild = scopes[scopes.Count - 1];
      IScope leftChild = scopes[0];

      if (rightChild.SubScopes.Count != 1) {
        throw new ArgumentException("Right child is only allowed to have one sub scope");
      }

      foreach (var variable in rightChild.SubScopes[0].Variables) {
        CurrentScope.Variables.Add(new Variable(variable.Name, variable.Value));
      }

      return leftChild.SubScopes.ToList();
    }
  }
}
